@@ -20,8 +20,12 @@ public:
 
 protected:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
+	std::string main_widget() const override
+	{
+		return "lastline";
+	}
 };
 
 } // namespace newsboat
