@@ -48,6 +48,12 @@ public:
 
 	void mark_pos_if_visible(unsigned int pos);
 
+protected:
+	std::string main_widget() const override
+	{
+		return "feeds";
+	}
+
 private:
 	void register_format_styles();
 
@@ -58,8 +64,8 @@ private:
 
 	int get_pos(unsigned int realidx);
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 
 	bool open_position_in_browser(unsigned int pos, bool interactive) const;
 
