@@ -39,8 +39,8 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void update_title(const std::string& working_directory);
 
 	void add_file(std::vector<file_system::FileSystemEntry>& id_at_position,
