@@ -29,29 +29,11 @@ HelpFormAction::HelpFormAction(View* vv,
 HelpFormAction::~HelpFormAction() {}
 
 bool HelpFormAction::process_operation(Operation op,
-	bool /* automatic */,
-	std::vector<std::string>* /* args */)
+	const std::vector<std::string>& /* args */,
+	BindingType /*bindingType*/)
 {
 	bool hardquit = false;
 	switch (op) {
-	case OP_SK_UP:
-		textview.scroll_up();
-		break;
-	case OP_SK_DOWN:
-		textview.scroll_down();
-		break;
-	case OP_SK_HOME:
-		textview.scroll_to_top();
-		break;
-	case OP_SK_END:
-		textview.scroll_to_bottom();
-		break;
-	case OP_SK_PGUP:
-		textview.scroll_page_up();
-		break;
-	case OP_SK_PGDOWN:
-		textview.scroll_page_down();
-		break;
 	case OP_QUIT:
 		quit = true;
 		break;
@@ -69,6 +51,9 @@ bool HelpFormAction::process_operation(Operation op,
 		do_redraw = true;
 		break;
 	default:
+		if (handle_textview_operations(textview, op)) {
+			break;
+		}
 		break;
 	}
 	if (hardquit) {
@@ -96,21 +81,24 @@ void HelpFormAction::prepare()
 
 		const auto descs = v->get_keymap()->get_keymap_descriptions(context);
 
-		std::string highlighted_searchphrase =
-			strprintf::fmt("<hl>%s</>", searchphrase);
 		std::vector<std::string> colors = utils::tokenize(
 				cfg->get_configvalue("search-highlight-colors"), " ");
 		set_value("highlight", make_colorstring(colors));
 		ListFormatter listfmt;
 
-		const unsigned int unbound_count = std::count_if(descs.begin(),
-		descs.end(), [](const KeyMapDesc& desc) {
-			return desc.key.length() == 0;
-		});
-		const unsigned int syskey_count = std::count_if(descs.begin(),
-		descs.end(), [](const KeyMapDesc& desc) {
-			return desc.flags & KM_SYSKEYS;
-		});
+		std::vector<KeyMapDesc> syskey_descriptions;
+		std::vector<KeyMapDesc> unbound_descriptions;
+		std::vector<KeyMapDesc> bound_descriptions;
+
+		for (const auto& desc : descs) {
+			if (desc.flags & KM_SYSKEYS) {
+				syskey_descriptions.push_back(desc);
+			} else if (desc.key.empty()) {
+				unbound_descriptions.push_back(desc);
+			} else {
+				bound_descriptions.push_back(desc);
+			}
+		}
 
 		const auto should_be_visible = [&](const KeyMapDesc& desc) {
 			return !apply_search
@@ -119,6 +107,7 @@ void HelpFormAction::prepare()
 				|| strcasestr(desc.desc.c_str(), searchphrase.c_str()) != nullptr;
 		};
 
+		std::string highlighted_searchphrase = strprintf::fmt("<hl>%s</>", searchphrase);
 		const auto apply_highlights = [&](const std::string& line) {
 			if (apply_search && searchphrase.length() > 0) {
 				return utils::replace_all(line, searchphrase, highlighted_searchphrase);
@@ -126,10 +115,7 @@ void HelpFormAction::prepare()
 			return line;
 		};
 
-		for (const auto& desc : descs) {
-			if (desc.key.length() == 0 || desc.flags & KM_SYSKEYS) {
-				continue;
-			}
+		for (const auto& desc : bound_descriptions) {
 			if (should_be_visible(desc)) {
 				auto line = strprintf::fmt("%-15s %-23s %s", desc.key, desc.cmd, desc.desc);
 				line = utils::quote_for_stfl(line);
@@ -138,15 +124,12 @@ void HelpFormAction::prepare()
 			}
 		}
 
-		if (syskey_count > 0) {
+		if (!syskey_descriptions.empty()) {
 			listfmt.add_line("");
 			listfmt.add_line(_("Generic bindings:"));
 			listfmt.add_line("");
 
-			for (const auto& desc : descs) {
-				if (!(desc.flags & KM_SYSKEYS)) {
-					continue;
-				}
+			for (const auto& desc : syskey_descriptions) {
 				if (should_be_visible(desc)) {
 					auto line = strprintf::fmt("%-15s %-23s %s", desc.key, desc.cmd, desc.desc);
 					line = utils::quote_for_stfl(line);
@@ -156,15 +139,12 @@ void HelpFormAction::prepare()
 			}
 		}
 
-		if (unbound_count > 0) {
+		if (!unbound_descriptions.empty()) {
 			listfmt.add_line("");
 			listfmt.add_line(_("Unbound functions:"));
 			listfmt.add_line("");
 
-			for (const auto& desc : descs) {
-				if (desc.key.length() > 0 || desc.flags & KM_SYSKEYS) {
-					continue;
-				}
+			for (const auto& desc : unbound_descriptions) {
 				if (should_be_visible(desc)) {
 					std::string line = strprintf::fmt("%-39s %s", desc.cmd, desc.desc);
 					line = utils::quote_for_stfl(line);
@@ -204,12 +184,11 @@ void HelpFormAction::init()
 	set_keymap_hints();
 }
 
-KeyMapHintEntry* HelpFormAction::get_keymap_hint()
+const std::vector<KeyMapHintEntry>& HelpFormAction::get_keymap_hint() const
 {
-	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Quit")},
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Quit")},
 		{OP_SEARCH, _("Search")},
-		{OP_CLEARFILTER, _("Clear")},
-		{OP_NIL, nullptr}
+		{OP_CLEARFILTER, _("Clear")}
 	};
 	return hints;
 }
