@@ -8,6 +8,7 @@
 #include <memory>
 #include <time.h>
 
+#include "curldatareceiver.h"
 #include "curlhandle.h"
 #include "utils.h"
 
@@ -17,18 +18,16 @@ namespace newsboat {
 
 typedef std::unique_ptr<json_object, decltype(*json_object_put)> JsonUptr;
 
-OcNewsApi::OcNewsApi(ConfigContainer* c)
+OcNewsApi::OcNewsApi(ConfigContainer& c)
 	: RemoteApi(c)
 {
-	server = cfg->get_configvalue("ocnews-url");
+	server = cfg.get_configvalue("ocnews-url");
 
 	if (server.empty())
 		LOG(Level::CRITICAL,
 			"OcNewsApi::OcNewsApi: No owncloud server set");
 }
 
-OcNewsApi::~OcNewsApi() {}
-
 bool OcNewsApi::authenticate()
 {
 	auth = retrieve_auth();
@@ -196,31 +195,23 @@ bool OcNewsApi::update_article_flags(const std::string& oldflags,
 	const std::string& newflags,
 	const std::string& guid)
 {
-	const auto feedIdStart = guid.find_first_of(":") + 1;
-	const auto feedIdEnd = guid.find_first_of("/");
-	const std::string feedId = guid.substr(feedIdStart, feedIdEnd - feedIdStart);
-
-	const std::string ocnewsGuid = guid.substr(guid.find_first_of("/") + 1);
-	const std::string guidHash = utils::md5hash(ocnewsGuid);
-
-	std::string query = strprintf::fmt("items/%s/%s", feedId, guidHash);
-
-	std::string star_flag = cfg->get_configvalue("ocnews-flag-star");
+	std::string star_flag = cfg.get_configvalue("ocnews-flag-star");
+	bool success = true;
 	if (star_flag.length() > 0) {
-		if (strchr(oldflags.c_str(), star_flag[0]) == nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) != nullptr) {
-			query += "/star";
-		} else if (strchr(oldflags.c_str(), star_flag[0]) != nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) == nullptr) {
-			query += "/unstar";
-		} else {
-			return true;
-		}
-	} else {
-		return true;
+		update_flag(oldflags, newflags, star_flag[0], [&](bool added) {
+			const auto feedIdStart = guid.find_first_of(':') + 1;
+			const auto feedIdEnd = guid.find_first_of('/');
+			const auto feedId = guid.substr(feedIdStart, feedIdEnd - feedIdStart);
+			const auto ocnewsGuid = guid.substr(feedIdEnd + 1);
+			const auto query = strprintf::fmt(
+					"items/%s/%s/%s",
+					feedId,
+					utils::md5hash(ocnewsGuid),
+					added ? "star" : "unstar");
+			success = this->query(query, nullptr, "{}");
+		});
 	}
-
-	return this->query(query, nullptr, "{}");
+	return success;
 }
 
 rsspp::Feed OcNewsApi::fetch_feed(const std::string& feed_id)
@@ -289,12 +280,17 @@ rsspp::Feed OcNewsApi::fetch_feed(const std::string& feed_id)
 			const auto type_ptr = json_object_get_string(type_obj);
 			const auto url_ptr = json_object_get_string(node);
 
-			if (type_ptr != nullptr) {
+			if (type_ptr != nullptr && url_ptr != nullptr) {
 				const std::string type = type_ptr;
-				if (utils::is_valid_podcast_type(type) && url_ptr != nullptr) {
-					item.enclosure_url = url_ptr;
-					item.enclosure_type = type;
+				const std::string url = url_ptr;
+				item.enclosures.push_back(
+				rsspp::Enclosure {
+					url,
+					type,
+					"",
+					"",
 				}
+				);
 			}
 		}
 
@@ -347,17 +343,6 @@ bool OcNewsApi::query(const std::string& query,
 
 	utils::set_common_curl_options(handle, cfg);
 
-	static auto write_fn =
-	[](void* buffer, size_t size, size_t nmemb, void* userp) {
-		std::string* pbuf = static_cast<std::string*>(userp);
-		pbuf->append(
-			static_cast<const char*>(buffer), size * nmemb);
-		return size * nmemb;
-	};
-	std::string buff;
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, *write_fn);
-	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &buff);
-
 	if (!post.empty()) {
 		curl_easy_setopt(handle.ptr(), CURLOPT_POST, 1);
 		curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, post.c_str());
@@ -370,6 +355,8 @@ bool OcNewsApi::query(const std::string& query,
 	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
 	curl_easy_setopt(handle.ptr(), CURLOPT_USERPWD, auth.c_str());
 
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(handle);
+
 	CURLcode res = curl_easy_perform(handle.ptr());
 
 	curl_slist_free_all(headers);
@@ -389,14 +376,15 @@ bool OcNewsApi::query(const std::string& query,
 			LOG(Level::CRITICAL,
 				"OcNewsApi::query: authentication error");
 		else {
-			std::string msg = "OcNewsApi::query: error ";
-			msg += response_code;
-			LOG(Level::CRITICAL, msg);
+			LOG(Level::CRITICAL,
+				"OcNewsApi::query: error %" PRIi64,
+				static_cast<int64_t>(response_code));
 		}
 		return false;
 	}
 
 	if (result) {
+		const std::string buff = curlDataReceiver->get_data();
 		*result = json_tokener_parse(buff.c_str());
 	}
 	return true;
