@@ -52,8 +52,8 @@ void SelectFormAction::handle_cmdline(const std::string& cmd)
 }
 
 bool SelectFormAction::process_operation(Operation op,
-	bool /* automatic */,
-	std::vector<std::string>* /* args */)
+	const std::vector<std::string>& /* args */,
+	BindingType /*bindingType*/)
 {
 	bool hardquit = false;
 	switch (op) {
