@@ -28,6 +28,7 @@
 #include <unordered_set>
 
 #include "config.h"
+#include "curldatareceiver.h"
 #include "curlhandle.h"
 #include "htmlrenderer.h"
 #include "logger.h"
@@ -191,14 +192,6 @@ std::string utils::get_command_output(const std::string& cmd)
 	return std::string(utils::bridged::get_command_output(cmd));
 }
 
-static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
-	void* userp)
-{
-	std::string* pbuf = static_cast<std::string*>(userp);
-	pbuf->append(static_cast<const char*>(buffer), size * nmemb);
-	return size * nmemb;
-}
-
 std::string utils::http_method_str(const HTTPMethod method)
 {
 	std::string str = "";
@@ -244,7 +237,7 @@ std::string utils::link_type_str(LinkType type)
 }
 
 std::string utils::retrieve_url(const std::string& url,
-	ConfigContainer* cfgcont,
+	ConfigContainer& cfgcont,
 	const std::string& authinfo,
 	const std::string* body,
 	const HTTPMethod method /* = GET */)
@@ -255,17 +248,15 @@ std::string utils::retrieve_url(const std::string& url,
 
 std::string utils::retrieve_url(const std::string& url,
 	CurlHandle& easyhandle,
-	ConfigContainer* cfgcont,
+	ConfigContainer& cfgcont,
 	const std::string& authinfo,
 	const std::string* body,
 	const HTTPMethod method /* = GET */)
 {
-	std::string buf;
-
 	set_common_curl_options(easyhandle, cfgcont);
 	curl_easy_setopt(easyhandle.ptr(), CURLOPT_URL, url.c_str());
-	curl_easy_setopt(easyhandle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(easyhandle.ptr(), CURLOPT_WRITEDATA, &buf);
+
+	auto curlDataReceiver = CurlDataReceiver::register_data_handler(easyhandle);
 
 	switch (method) {
 	case HTTPMethod::GET:
@@ -286,7 +277,7 @@ std::string utils::retrieve_url(const std::string& url,
 	}
 
 	if (!authinfo.empty()) {
-		const auto auth_method = cfgcont->get_configvalue("http-auth-method");
+		const auto auth_method = cfgcont.get_configvalue("http-auth-method");
 		curl_easy_setopt(easyhandle.ptr(), CURLOPT_HTTPAUTH, get_auth_method(auth_method));
 		curl_easy_setopt(easyhandle.ptr(), CURLOPT_USERPWD, authinfo.c_str());
 	}
@@ -305,6 +296,7 @@ std::string utils::retrieve_url(const std::string& url,
 	logprefix << "utils::retrieve_url(" << http_method_str(method) << " " << url << ")"
 		<< "[" << (body != nullptr ? body->c_str() : "-") << "]";
 
+	std::string buf = curlDataReceiver->get_data();
 	if (res != CURLE_OK) {
 		std::string errmsg(errbuf);
 		if (errmsg.empty()) {
@@ -386,6 +378,22 @@ std::string utils::replace_all(const std::string& str,
 	return output;
 }
 
+std::string utils::preserve_quotes(const std::string& str)
+{
+	std::string escaped_string = "";
+	std::vector<std::string> string_tokenized = tokenize_spaced(str, "'");
+	for (std::string string_chunk : string_tokenized) {
+		if (string_chunk[0] == '\'') {
+			for (size_t i = 0; i < string_chunk.length(); i++) {
+				escaped_string += "\\\'";
+			}
+		} else {
+			escaped_string += "'" + string_chunk + "'";
+		}
+	}
+	return escaped_string;
+}
+
 std::wstring utils::str2wstr(const std::string& str)
 {
 	const char* codeset = nl_langinfo(CODESET);
@@ -410,9 +418,9 @@ std::string utils::absolute_url(const std::string& url, const std::string& link)
 	return std::string(utils::bridged::absolute_url(url, link));
 }
 
-std::string utils::get_useragent(ConfigContainer* cfgcont)
+std::string utils::get_useragent(ConfigContainer& cfgcont)
 {
-	std::string ua_pref = cfgcont->get_configvalue("user-agent");
+	std::string ua_pref = cfgcont.get_configvalue("user-agent");
 	if (ua_pref.length() == 0) {
 		struct utsname buf;
 		uname(&buf);
@@ -522,68 +530,66 @@ std::string utils::quote_if_necessary(const std::string& str)
 	return std::string(utils::bridged::quote_if_necessary(str));
 }
 
-void utils::set_common_curl_options(CurlHandle& handle, ConfigContainer* cfg)
+void utils::set_common_curl_options(CurlHandle& handle, ConfigContainer& cfg)
 {
-	if (cfg) {
-		if (cfg->get_configvalue_as_bool("use-proxy")) {
-			const std::string proxy = cfg->get_configvalue("proxy");
-			if (proxy != "")
-				curl_easy_setopt(
-					handle.ptr(), CURLOPT_PROXY, proxy.c_str());
+	if (cfg.get_configvalue_as_bool("use-proxy")) {
+		const std::string proxy = cfg.get_configvalue("proxy");
+		if (proxy != "")
+			curl_easy_setopt(
+				handle.ptr(), CURLOPT_PROXY, proxy.c_str());
 
-			const std::string proxyauth =
-				cfg->get_configvalue("proxy-auth");
-			const std::string proxyauthmethod =
-				cfg->get_configvalue("proxy-auth-method");
-			if (proxyauth != "") {
-				curl_easy_setopt(handle.ptr(),
-					CURLOPT_PROXYAUTH,
-					get_auth_method(proxyauthmethod));
-				curl_easy_setopt(handle.ptr(),
-					CURLOPT_PROXYUSERPWD,
-					proxyauth.c_str());
-			}
-
-			const std::string proxytype =
-				cfg->get_configvalue("proxy-type");
-			if (proxytype != "") {
-				LOG(Level::DEBUG,
-					"utils::set_common_curl_options: "
-					"proxytype "
-					"= %s",
-					proxytype);
-				curl_easy_setopt(handle.ptr(),
-					CURLOPT_PROXYTYPE,
-					get_proxy_type(proxytype));
-			}
+		const std::string proxyauth =
+			cfg.get_configvalue("proxy-auth");
+		const std::string proxyauthmethod =
+			cfg.get_configvalue("proxy-auth-method");
+		if (proxyauth != "") {
+			curl_easy_setopt(handle.ptr(),
+				CURLOPT_PROXYAUTH,
+				get_auth_method(proxyauthmethod));
+			curl_easy_setopt(handle.ptr(),
+				CURLOPT_PROXYUSERPWD,
+				proxyauth.c_str());
 		}
 
-		const std::string useragent = utils::get_useragent(cfg);
-		curl_easy_setopt(handle.ptr(), CURLOPT_USERAGENT, useragent.c_str());
-
-		const unsigned int dl_timeout =
-			cfg->get_configvalue_as_int("download-timeout");
-		curl_easy_setopt(handle.ptr(), CURLOPT_TIMEOUT, dl_timeout);
-
-		const std::string cookie_cache =
-			cfg->get_configvalue("cookie-cache");
-		if (cookie_cache != "") {
+		const std::string proxytype =
+			cfg.get_configvalue("proxy-type");
+		if (proxytype != "") {
+			LOG(Level::DEBUG,
+				"utils::set_common_curl_options: "
+				"proxytype "
+				"= %s",
+				proxytype);
 			curl_easy_setopt(handle.ptr(),
-				CURLOPT_COOKIEFILE,
-				cookie_cache.c_str());
-			curl_easy_setopt(handle.ptr(),
-				CURLOPT_COOKIEJAR,
-				cookie_cache.c_str());
+				CURLOPT_PROXYTYPE,
+				get_proxy_type(proxytype));
 		}
-
-		curl_easy_setopt(handle.ptr(),
-			CURLOPT_SSL_VERIFYHOST,
-			cfg->get_configvalue_as_bool("ssl-verifyhost") ? 2 : 0);
-		curl_easy_setopt(handle.ptr(),
-			CURLOPT_SSL_VERIFYPEER,
-			cfg->get_configvalue_as_bool("ssl-verifypeer"));
 	}
 
+	const std::string useragent = utils::get_useragent(cfg);
+	curl_easy_setopt(handle.ptr(), CURLOPT_USERAGENT, useragent.c_str());
+
+	const unsigned int dl_timeout =
+		cfg.get_configvalue_as_int("download-timeout");
+	curl_easy_setopt(handle.ptr(), CURLOPT_TIMEOUT, dl_timeout);
+
+	const std::string cookie_cache =
+		cfg.get_configvalue("cookie-cache");
+	if (cookie_cache != "") {
+		curl_easy_setopt(handle.ptr(),
+			CURLOPT_COOKIEFILE,
+			cookie_cache.c_str());
+		curl_easy_setopt(handle.ptr(),
+			CURLOPT_COOKIEJAR,
+			cookie_cache.c_str());
+	}
+
+	curl_easy_setopt(handle.ptr(),
+		CURLOPT_SSL_VERIFYHOST,
+		cfg.get_configvalue_as_bool("ssl-verifyhost") ? 2 : 0);
+	curl_easy_setopt(handle.ptr(),
+		CURLOPT_SSL_VERIFYPEER,
+		cfg.get_configvalue_as_bool("ssl-verifypeer"));
+
 	curl_easy_setopt(handle.ptr(), CURLOPT_NOSIGNAL, 1);
 	// Accept all of curl's built-in encodings
 	curl_easy_setopt(handle.ptr(), CURLOPT_ACCEPT_ENCODING, "");
